/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ClassAnnotationMetadata.java 2059 2007-11-22 17:22:33Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.annotations.metadata;

import static javax.ejb.TransactionAttributeType.REQUIRED;
import static javax.ejb.TransactionManagementType.CONTAINER;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.ejb.ApplicationException;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagementType;

import org.ow2.easybeans.deployment.annotations.ClassType;
import org.ow2.easybeans.deployment.annotations.InterceptorType;
import org.ow2.easybeans.deployment.annotations.JClassInterceptor;
import org.ow2.easybeans.deployment.annotations.JField;
import org.ow2.easybeans.deployment.annotations.JMethod;
import org.ow2.easybeans.deployment.annotations.exceptions.InterceptorsValidationException;
import org.ow2.easybeans.deployment.annotations.impl.JAnnotationResource;
import org.ow2.easybeans.deployment.annotations.impl.JCommonBean;
import org.ow2.easybeans.deployment.annotations.impl.JEjbEJB;
import org.ow2.easybeans.deployment.annotations.impl.JInterceptors;
import org.ow2.easybeans.deployment.annotations.impl.JLocal;
import org.ow2.easybeans.deployment.annotations.impl.JMessageDriven;
import org.ow2.easybeans.deployment.annotations.impl.JRemote;
import org.ow2.easybeans.deployment.annotations.impl.JStateful;
import org.ow2.easybeans.deployment.annotations.impl.JStateless;
import org.ow2.easybeans.deployment.annotations.impl.JavaxPersistenceContext;
import org.ow2.easybeans.deployment.annotations.impl.JavaxPersistenceUnit;
import org.ow2.easybeans.deployment.annotations.metadata.interfaces.IAnnotationSecurityPermitAll;
import org.ow2.easybeans.deployment.annotations.metadata.interfaces.IAnnotationSecurityRolesAllowed;
import org.ow2.easybeans.deployment.annotations.metadata.interfaces.IEJBInterceptors;
import org.ow2.easybeans.deployment.annotations.metadata.interfaces.IInterceptorExcludeDefaultInterceptors;
import org.ow2.easybeans.deployment.annotations.metadata.interfaces.ITransactionAttribute;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;


/**
 * This class represents the annotation metadata of a Bean.<br>
 * From this class, we can access to all methods of a bean with its associated information.
 * @author Florent Benoit
 */
public class ClassAnnotationMetadata extends CommonAnnotationMetadata implements ITransactionAttribute, IEJBInterceptors,
        IAnnotationSecurityRolesAllowed, IAnnotationSecurityPermitAll, IInterceptorExcludeDefaultInterceptors {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(ClassAnnotationMetadata.class);

    /**
     * List of method annotations metadata.
     */
    private Map<JMethod, MethodAnnotationMetadata> methodsAnnotationMetadata = null;

    /**
     * List of field annotations metadata.
     */
    private Map<JField, FieldAnnotationMetadata> fieldsAnnotationMetadata = null;

    /**
     * Parent meta data.
     */
    private EjbJarAnnotationMetadata ejbJarAnnotationMetadata = null;

    /**
     * List of local interfaces.
     */
    private JLocal jLocal = null;

    /**
     * List of remote interfaces.
     */
    private JRemote jRemote = null;

    /**
     * CommonBean description.
     */
    private JCommonBean jCommonBean = null;

    /**
     * Message Driven attribute.
     */
    private JMessageDriven jMessageDriven = null;

    /**
     * Stateless attribute.
     */
    private JStateless jStateless = null;

    /**
     * Stateful attribute.
     */
    private JStateful jStateful = null;

    /**
     * Ordered interceptors. If this flag is true it means that
     * interceptor-binding has been used in DD with an interceptor-order
     * element.<br/>
     * Also, this flag is used to check that the ordering is done only once.
     */
    private boolean orderedInterceptors = false;

    /**
     * Local Home.
     */
    private String localHome = null;

    /**
     * Remote Home.
     */
    private String remoteHome = null;

    /**
     * List of annotation interceptors.
     */
    private JInterceptors annotationInterceptors = null;

    /**
     * EasyBeans global interceptors.<br>
     * These interceptors correspond to a list of interceptors
     * that need to be present first on all methods.
     */
    private List<JClassInterceptor> globalEasyBeansInterceptors = null;

    /**
     * User interceptors. These interceptors correspond to a list of Interceptor that user has specified in its bean class.
     * It is the interceptors defined in interceptor classes, not the bean class itself.
     * Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt;
     */
    private Map<InterceptorType, List<JClassInterceptor>> externalUserInterceptors = null;

    /**
     * User interceptors. These interceptors correspond to a list of Interceptor that user has specified in the bean class.
     * It is not defined in separated classes.
     * Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt;
     */
    private Map<InterceptorType, List<JClassInterceptor>> internalUserInterceptors = null;


    /**
     * Transaction management type (default = container).
     */
    private TransactionManagementType transactionManagementType = CONTAINER;

    /**
     * Transaction attribute type (default = required).
     */
    private TransactionAttributeType transactionAttributeType = REQUIRED;

    /**
     * Application exception annotation.
     */
    private ApplicationException applicationException = null;

    /**
     * Superclass name.
     */
    private String superName = null;

    /**
     * Interfaces of this clas.
     */
    private String[] interfaces = null;

    /**
     * The type of the class.
     * @see ClassType
     */
    private ClassType classType = null;

    /**
     * Name of the class associated to this metadata.
     */
    private String className = null;

    /**
     * List of &#64;{@link javax.interceptor.AroundInvoke} methods on this
     * class (should be only one per class, validating occurs after).
     */
    private List<MethodAnnotationMetadata> aroundInvokeMethodsMetadata = null;

    /**
     * Object representing &#64;{@link javax.ejb.EJBs} annotation.
     */
    private List<JEjbEJB> jEjbEJBs = null;

    /**
     * Object representing &#64;{@link javax.annotation.Resources} annotation.
     */
    private List<JAnnotationResource> jAnnotationResources = null;

    /**
     * Object representing &#64;{@link javax.persistence.PersistenceContext} annotation.
     */
    private List<JavaxPersistenceContext> javaxPersistencePersistenceContexts = null;

    /**
     * Object representing &#64;{@link javax.persistence.PersistenceUnit} annotation.
     */
    private List<JavaxPersistenceUnit> javaxPersistencePersistenceUnits = null;

    /**
     * Methods used for &#64;{@link javax.annotation.PostConstruct} on this
     * class (only one per class but may be defined in super classes).
     */
    private LinkedList<MethodAnnotationMetadata> postConstructMethodsMetadata = null;

    /**
     * Methods used for &#64;{@link javax.annotation.PreDestroy} on this class
     * (only one per class but may be defined in super classes).
     */
    private LinkedList<MethodAnnotationMetadata> preDestroyMethodsMetadata = null;

    /**
     * Methods used for &#64;{@link javax.ejb.PostActivate} on this class (only
     * one per class but may be defined in super classes).
     */
    private LinkedList<MethodAnnotationMetadata> postActivateMethodsMetadata = null;

    /**
     * Methods used for &#64;{@link javax.ejb.PrePassivate} on this class (only
     * one per class but may be defined in super classes).
     */
    private LinkedList<MethodAnnotationMetadata> prePassivateMethodsMetadata = null;

    /**
     * Is that the class represented by this metadata has already been modified ?
     */
    private boolean modified = false;

    /**
     * List of roles that are declared on this class.
     */
    private List<String> declareRoles = null;

    /**
     * List of roles that are allowed on this class/bean.
     */
    private List<String> rolesAllowed = null;

    /**
     * This class/bean has the permitAll annotation.
     */
    private boolean permitAll = false;

    /**
     * The run-as security role (if any).
     */
    private String runAs = null;

    /**
     * &#64;{@link javax.interceptor.ExcludeDefaultInterceptors} method ?
     */
    private boolean excludeDefaultInterceptors = false;

    /**
     * Name of the remote interface (used in EJB 2.1 DD).
     */
    private String remoteInterface = null;

    /**
     * Constructor.
     * @param className name of the class associated to these metadatas.
     * @param ejbJarAnnotationMetadata parent metadata object.
     */
    public ClassAnnotationMetadata(final String className, final EjbJarAnnotationMetadata ejbJarAnnotationMetadata) {
        this.className = className;
        this.methodsAnnotationMetadata = new HashMap<JMethod, MethodAnnotationMetadata>();
        this.fieldsAnnotationMetadata = new HashMap<JField, FieldAnnotationMetadata>();
        this.ejbJarAnnotationMetadata = ejbJarAnnotationMetadata;
        this.postConstructMethodsMetadata = new LinkedList<MethodAnnotationMetadata>();
        this.preDestroyMethodsMetadata = new LinkedList<MethodAnnotationMetadata>();
        this.postActivateMethodsMetadata = new LinkedList<MethodAnnotationMetadata>();
        this.prePassivateMethodsMetadata = new LinkedList<MethodAnnotationMetadata>();
    }

    /**
     * @return name of the bean (associated to this metadata).
     */
    public String getClassName() {
        return className;
    }

    /**
     * Add method annotation metadata for a given Bean.
     * @param methodAnnotationMetadata metadata of a method.
     */
    public void addMethodAnnotationMetadata(final MethodAnnotationMetadata methodAnnotationMetadata) {
        JMethod key = methodAnnotationMetadata.getJMethod();
        // No check: the same method can be added from the super class (ie : super = private, current = private)
        // So it means that there is no overriding
        methodsAnnotationMetadata.put(key, methodAnnotationMetadata);
    }

    /**
     * @param jMethod key of the map of methods annotations.
     * @return method annotation metadata of a given method.
     */
    public MethodAnnotationMetadata getMethodAnnotationMetadata(final JMethod jMethod) {
        return methodsAnnotationMetadata.get(jMethod);
    }

    /**
     * @param methodName the name of the method to search
     * @return list of method annotation metadata for the given method name
     */
    public List<MethodAnnotationMetadata> searchMethodAnnotationMetadata(final String methodName) {
        if (methodName == null) {
            throw new IllegalArgumentException("Cannot search null methods");
        }
        List<MethodAnnotationMetadata> list = new ArrayList<MethodAnnotationMetadata>();
        for (MethodAnnotationMetadata method : methodsAnnotationMetadata.values()) {
            if (methodName.equals(method.getMethodName())) {
                list.add(method);
            }
        }
        return list;
    }

    /**
     * @param fieldName the name of the field to search
     * @return list of field annotation metadata for the given field name
     */
    public List<FieldAnnotationMetadata> searchFieldAnnotationMetadata(final String fieldName) {
        if (fieldName == null) {
            throw new IllegalArgumentException("Cannot search null fields");
        }
        List<FieldAnnotationMetadata> list = new ArrayList<FieldAnnotationMetadata>();
        for (FieldAnnotationMetadata field : fieldsAnnotationMetadata.values()) {
            if (fieldName.equals(field.getFieldName())) {
                list.add(field);
            }
        }
        return list;
    }



    /**
     * Get collections of methods annotation metadata.
     * @return collections of methods annotation metadata.
     */
    public Collection<MethodAnnotationMetadata> getMethodAnnotationMetadataCollection() {
        return methodsAnnotationMetadata.values();
    }


    /**
     * Add field annotation metadata for a given Bean.
     * @param fieldAnnotationMetadata metadata of a field.
     */
    public void addFieldAnnotationMetadata(final FieldAnnotationMetadata fieldAnnotationMetadata) {
        JField key = fieldAnnotationMetadata.getJField();
        // already exists ?
        if (fieldsAnnotationMetadata.containsKey(key)) {
            String msg = logger.getI18n().getMessage("BeanAnnotationMetadata.addFieldAnnotationMetadata.alreadyPresent", key);
            logger.debug(msg);
            throw new IllegalStateException(msg);
        }
        fieldsAnnotationMetadata.put(key, fieldAnnotationMetadata);
    }

    /**
     * @param jField key of the map of fields annotations.
     * @return field annotation metadata of a given method.
     */
    public FieldAnnotationMetadata getFieldAnnotationMetadata(final JField jField) {
        return fieldsAnnotationMetadata.get(jField);
    }

    /**
     * Get collections of fields annotation metadata.
     * @return collections of fields annotation metadata.
     */
    public Collection<FieldAnnotationMetadata> getFieldAnnotationMetadataCollection() {
        return fieldsAnnotationMetadata.values();
    }


    /**
     * Sets the local interfaces of this class.
     * @param jLocal list of interfaces.
     */
    public void setLocalInterfaces(final JLocal jLocal) {
        this.jLocal = jLocal;
    }

    /**
     * Sets the remote interfaces of this class.
     * @param jRemote list of interfaces.
     */
    public void setRemoteInterfaces(final JRemote jRemote) {
        this.jRemote = jRemote;
    }


    /**
     * @return the local interfaces of this class.
     */
    public JLocal getLocalInterfaces() {
       return jLocal;
    }

    /**
     * @return the remote interfaces of this class.
     */
    public JRemote getRemoteInterfaces() {
        return jRemote;
    }

      /**
     * @return true if the class is a stateless class
     */
    public boolean isStateless() {
        return (classType != null && classType == ClassType.STATELESS);
    }

    /**
     * @return true if the class is a stateful class
     */
    public boolean isStateful() {
        return (classType != null && classType == ClassType.STATEFUL);
    }

    /**
     * @return true if the class is a session bean class
     */
    public boolean isSession() {
        return (classType != null && (classType == ClassType.STATELESS || classType == ClassType.STATEFUL));
    }

    /**
     * @return true if the class is an MDB class
     */
    public boolean isMdb() {
        return (classType != null && classType == ClassType.MDB);
    }

    /**
     * Sets the type of this class.
     * @param cType a type from enum class ClassType.
     * @see org.ow2.easybeans.deployment.annotations.ClassType
     */
    public void setClassType(final ClassType cType) {
        this.classType = cType;
    }

    /**
     * @return Message driven attribute.
     */
    public JMessageDriven getJMessageDriven() {
        return jMessageDriven;
    }

    /**
     * Sets the message driven bean object.
     * @param messageDriven attributes of message driven bean.
     */
    public void setJMessageDriven(final JMessageDriven messageDriven) {
        jMessageDriven = messageDriven;
    }


    /**
     * @return string representation.
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        // classname
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));
        sb.append("[\n");

        // Add super class toString()
        sb.append(super.toString());

        // Class name
        concatStringBuilder("className", className, sb);

        // superclass name
        concatStringBuilder("superName", superName, sb);

        // interfaces
        concatStringBuilder("interfaces", interfaces, sb);

        // classType
        concatStringBuilder("classType", classType, sb);

        // jLocal
        concatStringBuilder("jLocal", jLocal, sb);

        // aroundInvokeMethodMetadatas
        concatStringBuilder("aroundInvokeMethodsMetadata", aroundInvokeMethodsMetadata, sb);

        // jRemote
        concatStringBuilder("jRemote", jRemote, sb);

        // jMessageDriven
        concatStringBuilder("jMessageDriven", jMessageDriven, sb);

        // remoteHome
        concatStringBuilder("remoteHome", remoteHome, sb);

        // localHome
        concatStringBuilder("localHome", localHome, sb);

        // transactionManagementType
        concatStringBuilder("transactionManagementType", transactionManagementType, sb);

        // transactionAttributeType
        concatStringBuilder("transactionAttributeType", transactionAttributeType, sb);

        // annotation Interceptors
        concatStringBuilder("annotationInterceptors", annotationInterceptors, sb);

        // jEjbEJBs
        concatStringBuilder("jAnnotationEJBs", jEjbEJBs, sb);

        // jAnnotationResources
        concatStringBuilder("jAnnotationResources", jAnnotationResources, sb);

        // javaxPersistencePersistenceContexts
        concatStringBuilder("javaxPersistencePersistenceContexts", javaxPersistencePersistenceContexts, sb);

        // javaxPersistencePersistenceUnits
        concatStringBuilder("javaxPersistencePersistenceUnits", javaxPersistencePersistenceUnits, sb);


        // Methods
        for (MethodAnnotationMetadata methodAnnotationMetadata : getMethodAnnotationMetadataCollection()) {
            concatStringBuilder("methods", methodAnnotationMetadata, sb);
        }

        // modified
        concatStringBuilder("modified", Boolean.valueOf(modified), sb);

        // excludeDefaultInterceptors
        concatStringBuilder("excludeDefaultInterceptors", Boolean.valueOf(excludeDefaultInterceptors), sb);

        // declareRoles
        concatStringBuilder("declareRoles", declareRoles, sb);

        // rolesAllowed
        concatStringBuilder("rolesAllowed", rolesAllowed, sb);

        // permitAll
        concatStringBuilder("permitAll", Boolean.valueOf(permitAll), sb);

        // runAs
        concatStringBuilder("runAs", runAs, sb);

        sb.append("]");
        return sb.toString();
    }

    /**
     * @return the &#64;{@link javax.ejb.RemoteHome} class name.
     */
    public String getRemoteHome() {
        return remoteHome;
    }

    /**
     * Sets the &#64;{@link javax.ejb.RemoteHome} class name.
     * @param remoteHome the class name.
     */
    public void setRemoteHome(final String remoteHome) {
        this.remoteHome = remoteHome;
    }

    /**
     * @return the &#64;{@link javax.ejb.LocalHome} class name.
     */
    public String getLocalHome() {
        return localHome;
    }

    /**
     * Sets the &#64;{@link javax.ejb.LocalHome} class name.
     * @param localHome the class name.
     */
    public void setLocalHome(final String localHome) {
        this.localHome = localHome;
    }

    /**
     * @return transaction management type from @see TransactionManagementType.
     */
    public TransactionManagementType getTransactionManagementType() {
        return transactionManagementType;
    }

    /**
     * Sets transaction management type.
     * @see javax.ejb.TransactionManagementType
     * @param transactionManagementType value.
     *      (BEAN, CONTAINER)
     */
    public void setTransactionManagementType(final TransactionManagementType transactionManagementType) {
        this.transactionManagementType = transactionManagementType;
    }

    /**
     * @return transaction Attribute type.
     * @see javax.ejb.TransactionAttributeType
     */
    public TransactionAttributeType getTransactionAttributeType() {
        return transactionAttributeType;
    }

    /**
     * Set Transaction Attribute Type.
     * @see javax.ejb.TransactionAttributeType
     * @param transactionAttributeType the type of transaction.
     */
    public void setTransactionAttributeType(final TransactionAttributeType transactionAttributeType) {
        this.transactionAttributeType = transactionAttributeType;
    }

    /**
     * @return object representing list of &#64;{@link javax.interceptor.Interceptors}.
     */
    public JInterceptors getAnnotationInterceptors() {
        return annotationInterceptors;
    }

    /**
     * Sets the object representing the &#64;{@link javax.interceptor.Interceptors} annotation.
     * @param annotationInterceptors list of classes
     */
    public void setAnnotationsInterceptors(final JInterceptors annotationInterceptors) {
        this.annotationInterceptors = annotationInterceptors;
    }

    /**
     * @return the &#64;{@link javax.ejb.ApplicationException} annotation.
     */
    public ApplicationException getApplicationException() {
        return applicationException;
    }

    /**
     * Sets the object representing the &#64;{@link javax.ejb.ApplicationException} annotation.
     * @param applicationException object representation
     */
    public void setApplicationException(final ApplicationException applicationException) {
        this.applicationException = applicationException;
    }

    /**
     * @return true if the classs is a Bean
     */
    public boolean isBean() {
        return isStateless() || isStateful() || isMdb();
    }

    /**
     * @return array of interfaces name.
     */
    public String[] getInterfaces() {
        return interfaces;
    }

    /**
     * Sets the interfaces of this class.
     * @param interfaces name of interfaces.
     */
    public void setInterfaces(final String[] interfaces) {
        this.interfaces = interfaces;
    }

    /**
     * @return the super class name.
     */
    public String getSuperName() {
        return superName;
    }

    /**
     * Sets the super class name.
     * @param superName name of the super class.
     */
    public void setSuperName(final String superName) {
        this.superName = superName;
    }

    /**
     * @return parent metadata object.
     */
    public EjbJarAnnotationMetadata getEjbJarAnnotationMetadata() {
        return ejbJarAnnotationMetadata;
    }

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt; (interceptor classes)
     * of user interceptors that enhancer will use.
     */
    public Map<InterceptorType, List<JClassInterceptor>> getExternalUserEasyBeansInterceptors() {
        return externalUserInterceptors;
    }

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined outside the bean class (interceptor classes).
     * @param externalUserInterceptors list of interceptors that enhancer will use.
     */
    public void setExternalUserInterceptors(final Map<InterceptorType, List<JClassInterceptor>> externalUserInterceptors) {
        this.externalUserInterceptors = externalUserInterceptors;
    }


    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt; (bean classes)
     * of user interceptors that enhancer will use.
     */
    public Map<InterceptorType, List<JClassInterceptor>> getInternalUserEasyBeansInterceptors() {
        return internalUserInterceptors;
    }

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined in bean class.
     * @param internalUserInterceptors list of interceptors that enhancer will use.
     */
    public void setInternalUserInterceptors(final Map<InterceptorType, List<JClassInterceptor>> internalUserInterceptors) {
        this.internalUserInterceptors = internalUserInterceptors;
    }

    /**
     * @return list of global interceptors that enhancer will use. (ie : ENC)
     */
    public List<JClassInterceptor> getGlobalEasyBeansInterceptors() {
        return globalEasyBeansInterceptors;
    }

    /**
     * Sets the list of global interceptors that enhancers will use.
     * @param globalEasyBeansInterceptors list of interceptors that enhancer will use.
     */
    public void setGlobalEasyBeansInterceptors(final List<JClassInterceptor> globalEasyBeansInterceptors) {
        this.globalEasyBeansInterceptors = globalEasyBeansInterceptors;
    }

    /**
     * @return the method metadata with annotation &#64;{@link javax.interceptor.AroundInvoke}.
     */
    public boolean isAroundInvokeMethodMetadata() {
        return (aroundInvokeMethodsMetadata != null);
    }


    /**
     * @return the list of methods metadata with annotation &#64;{@link javax.interceptor.AroundInvoke}.
     */
    public List<MethodAnnotationMetadata> getAroundInvokeMethodMetadatas() {
        return aroundInvokeMethodsMetadata;
    }

    /**
     * Add a &#64;{@link javax.interceptor.AroundInvoke} method of this class.
     * @param aroundInvokeMethodMetadata the method.
     */
    public void addAroundInvokeMethodMetadata(final MethodAnnotationMetadata aroundInvokeMethodMetadata) {
        if (aroundInvokeMethodsMetadata == null) {
            this.aroundInvokeMethodsMetadata = new ArrayList<MethodAnnotationMetadata>();
        }
        aroundInvokeMethodsMetadata.add(aroundInvokeMethodMetadata);
    }

    /**
     * @return the methods metadata with annotation &#64;{@link javax.annotation.PostConstruct}.
     */
    public LinkedList<MethodAnnotationMetadata> getPostConstructMethodsMetadata() {
        return postConstructMethodsMetadata;
    }

    /**
     * Adds a &#64;{@link javax.annotation.PostConstruct} method of this class.
     * @param postConstructMethodMetadata the method.
     */
    public void addPostConstructMethodMetadata(final MethodAnnotationMetadata postConstructMethodMetadata) {
      checkLifeCycleDuplicate(postConstructMethodMetadata, InterceptorType.POST_CONSTRUCT, getPostConstructMethodsMetadata());
      this.postConstructMethodsMetadata.addFirst(postConstructMethodMetadata);
    }

    /**
     * Checks that only method at one level of a class is present.
     * @param postConstructMethodMetadata method to check
     * @param itcType the type of interceptor (used for the error)
     * @param existingList current list of methods
     */
    private void checkLifeCycleDuplicate(final MethodAnnotationMetadata postConstructMethodMetadata,
            final InterceptorType itcType, final List<MethodAnnotationMetadata> existingList) {
        // First case : not inherited
        ClassAnnotationMetadata wantToAddClassMetadata = postConstructMethodMetadata.getClassAnnotationMetadata();
        if (postConstructMethodMetadata.isInherited()) {
            wantToAddClassMetadata = postConstructMethodMetadata.getOriginalClassAnnotationMetadata();
        }
        for (MethodAnnotationMetadata method : existingList) {
            ClassAnnotationMetadata compareMetaData;
            if (method.isInherited()) {
                compareMetaData = method.getOriginalClassAnnotationMetadata();
            } else {
                compareMetaData = method.getClassAnnotationMetadata();
            }
            if (compareMetaData.equals(wantToAddClassMetadata)) {
                throw new InterceptorsValidationException("Class " + getClassName()
                        + " has already a " + itcType + " method which is "
                        + method.getMethodName() + ", cannot set new method "
                        + postConstructMethodMetadata.getMethodName());
            }
        }
    }


    /**
     * @return the methods metadata with annotation &#64;{@link javax.annotation.PreDestroy}.
     */
    public LinkedList<MethodAnnotationMetadata> getPreDestroyMethodsMetadata() {
        return preDestroyMethodsMetadata;
    }

    /**
     * Adds a &#64;{@link javax.annotation.PreDestroy} method of this class.
     * @param preDestroyMethodMetadata the method.
     */
    public void addPreDestroyMethodMetadata(final MethodAnnotationMetadata preDestroyMethodMetadata) {
        checkLifeCycleDuplicate(preDestroyMethodMetadata, InterceptorType.PRE_DESTROY, getPreDestroyMethodsMetadata());
        this.preDestroyMethodsMetadata.addFirst(preDestroyMethodMetadata);
    }


    /**
     * @return the methods metadata with annotation &#64;{@link javax.ejb.PostActivate}.
     */
    public LinkedList<MethodAnnotationMetadata> getPostActivateMethodsMetadata() {
        return postActivateMethodsMetadata;
    }

    /**
     * Adds a &#64;{@link javax.ejb.PostActivate} method of this class.
     * @param postActivateMethodMetadata the method.
     */
    public void addPostActivateMethodMetadata(final MethodAnnotationMetadata postActivateMethodMetadata) {
        checkLifeCycleDuplicate(postActivateMethodMetadata, InterceptorType.POST_ACTIVATE, getPostActivateMethodsMetadata());
        this.postActivateMethodsMetadata.addFirst(postActivateMethodMetadata);
    }


    /**
     * @return the method metadata with annotation &#64;{@link javax.ejb.PrePassivate}.
     */
    public LinkedList<MethodAnnotationMetadata> getPrePassivateMethodsMetadata() {
        return prePassivateMethodsMetadata;
    }

    /**
     * Adds a &#64;{@link javax.ejb.PrePassivate} method of this class.
     * @param prePassivateMethodMetadata the method.
     */
    public void addPrePassivateMethodMetadata(final MethodAnnotationMetadata prePassivateMethodMetadata) {
        checkLifeCycleDuplicate(prePassivateMethodMetadata, InterceptorType.PRE_PASSIVATE, getPrePassivateMethodsMetadata());
        this.prePassivateMethodsMetadata.addFirst(prePassivateMethodMetadata);
    }


    /**
     * Is that this class is an interceptor class ?
     * @return true if it the case, else false.
     */
    public boolean isInterceptor() {
        return (aroundInvokeMethodsMetadata != null && aroundInvokeMethodsMetadata.size() > 0)
        || (postConstructMethodsMetadata != null && postConstructMethodsMetadata.size() > 0)
        || (preDestroyMethodsMetadata != null && preDestroyMethodsMetadata.size() > 0)
        || (prePassivateMethodsMetadata != null && prePassivateMethodsMetadata.size() > 0)
        || (postActivateMethodsMetadata != null && postActivateMethodsMetadata.size() > 0);
    }


    /**
     * @return jEjbEJBs list representing &#64;{@link javax.ejb.EJBs} annotation.
     */
    public List<JEjbEJB> getJEjbEJBs() {
        return jEjbEJBs;
    }

    /**
     * Set JEjbEJBs object.
     * @param jEjbEJBs list representing javax.ejb.EJBs annotation.
     */
    public void setJEjbEJBs(final List<JEjbEJB> jEjbEJBs) {
        this.jEjbEJBs = jEjbEJBs;
    }


    /**
     * @return JAnnotationResources list representing &#64;{@link javax.annotation.Resources} annotation.
     */
    public List<JAnnotationResource> getJAnnotationResources() {
        return jAnnotationResources;
    }

    /**
     * Sets JAnnotationResources object.
     * @param jAnnotationResources list representing javax.annotation.Resources annotation.
     */
    public void setJAnnotationResources(final List<JAnnotationResource> jAnnotationResources) {
        this.jAnnotationResources = jAnnotationResources;
    }

    /**
     * @return javaxPersistencePersistenceContexts list representing &#64;{@link javax.persistence.PersistenceContexts}
     *         annotation.
     */
    public List<JavaxPersistenceContext> getJavaxPersistencePersistenceContexts() {
        return javaxPersistencePersistenceContexts;
    }

    /**
     * Sets JavaxPersistencePersistenceContexts object.
     * @param javaxPersistencePersistenceContexts list representing &#64;{@link javax.persistence.PersistenceContexts} annotation.
     */
    public void setJavaxPersistencePersistenceContexts(final List<JavaxPersistenceContext> javaxPersistencePersistenceContexts) {
        this.javaxPersistencePersistenceContexts = javaxPersistencePersistenceContexts;
    }

    /**
     * @return javaxPersistencePersistenceUnits list representing &#64;{@link javax.persistence.PersistenceUnits} annotation.
     */
    public List<JavaxPersistenceUnit> getJavaxPersistencePersistenceUnits() {
        return javaxPersistencePersistenceUnits;
    }

    /**
     * Sets setJavaxPersistencePersistenceUnits object.
     * @param javaxPersistencePersistenceUnits list representing &#64;{@link javax.persistence.PersistenceUnits} annotation.
     */
    public void setJavaxPersistencePersistenceUnits(final List<JavaxPersistenceUnit> javaxPersistencePersistenceUnits) {
        this.javaxPersistencePersistenceUnits = javaxPersistencePersistenceUnits;
    }

    /**
     * @return the attributes for a Stateless/Stateful/MDB
     */
    public JCommonBean getJCommonBean() {
        return jCommonBean;
    }

    /**
     * Sets the attributes for a Stateless/Stateful/MDB.
     * @param commonBean the attributes
     */
    public void setJCommonBean(final JCommonBean commonBean) {
        // ensure that there is a bean name
        String ejbName = commonBean.getName();
        if (ejbName == null || "".equals(ejbName)) {
            // if not set, it has to be the unqualified name of the bean class. (simplified spec : 10.1.1)
            commonBean.setName(className.substring(className.lastIndexOf("/") + 1));
        }
        jCommonBean = commonBean;
    }

    /**
     * @return the attributes for a Stateful
     */
    public JStateful getJStateful() {
        return jStateful;
    }

    /**
     * Sets the attributes for a Stateful.
     * @param jStateful the attributes
     */
    public void setJStateful(final JStateful jStateful) {
        this.jStateful = jStateful;
    }

    /**
     * @return the attributes for a Stateless
     */
    public JStateless getJStateless() {
        return jStateless;
    }

    /**
     * Sets the attributes for a Stateless.
     * @param jStateless the attributes
     */
    public void setJStateless(final JStateless jStateless) {
        this.jStateless = jStateless;
    }

    /**
     * @return true if the class has been modified.
     */
    public boolean wasModified() {
        return modified;
    }

    /**
     * Defines that this class has been modified.
     */
    public void setModified() {
        this.modified = true;
    }

    /**
     * Sets the list of roles declared on this class.
     * @param declareRoles the list of roles.
     */
    public void setDeclareRoles(final List<String> declareRoles) {
        this.declareRoles = declareRoles;
    }

    /**
     * @return the list of roles declared on this class.
     */
    public List<String> getDeclareRoles() {
        return declareRoles;
    }

    /**
     * Set the list of roles allowed on this class/method.
     * @param rolesAllowed the list of roles.
     */
    public void setRolesAllowed(final List<String> rolesAllowed) {
        this.rolesAllowed = rolesAllowed;
    }

    /**
     * @return the list of roles allowed on this class/method.
     */
    public List<String> getRolesAllowed() {
        return rolesAllowed;
    }

    /**
     * This class has PermitAll annotation.
     * @param permitAll the boolean value.
     */
    public void setPermitAll(final boolean permitAll) {
        this.permitAll = permitAll;
    }

    /**
     * @return true if PermitAll annotation.
     */
    public boolean hasPermitAll() {
        return permitAll;
    }

    /**
     * Set the value of the run-as property.
     * @param runAs the run-as property.
     */
    public void setRunAs(final String runAs) {
        this.runAs = runAs;
    }

    /**
     * @return the name of the security-role of the run-as element.
     */
    public String getRunAs() {
        return runAs;
    }


    /**
     * @return true if this class won't use default interceptors.
     */
    public boolean isExcludedDefaultInterceptors() {
        return excludeDefaultInterceptors;
    }

    /**
     * Flag this class as a class which exclude default interceptors.
     * @param excludeDefaultInterceptors true if this class is a class which exclude default interceptors.
     */
    public void setExcludeDefaultInterceptors(final boolean excludeDefaultInterceptors) {
        this.excludeDefaultInterceptors = excludeDefaultInterceptors;
    }

    /**
     * @return true if the interceptors have been ordered for this class.
     */
    public boolean isOrderedInterceptors() {
        return orderedInterceptors;
    }

    /**
     * Sets flag to true if interceptors are ordered.
     * @param orderedInterceptors boolean with true/false.
     */
    public void setOrderedInterceptors(final boolean orderedInterceptors) {
        this.orderedInterceptors = orderedInterceptors;
    }

    /**
     * Set the value of the remote Interface property.
     * @param remoteInterface the remote Interface property.
     */
    public void setRemoteInterface(final String remoteInterface) {
        this.remoteInterface = remoteInterface;
    }

    /**
     * @return the name of the remote interface.
     */
    public String getRemoteInterface() {
        return remoteInterface;
    }

}
