/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EjbJarAnnotationMetadata.java 2059 2007-11-22 17:22:33Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.annotations.metadata;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ejb.ApplicationException;

import org.ow2.easybeans.deployment.annotations.InterceptorType;
import org.ow2.easybeans.deployment.annotations.JClassInterceptor;
import org.ow2.easybeans.deployment.annotations.impl.JApplicationException;
import org.ow2.easybeans.deployment.annotations.impl.JCommonBean;
import org.ow2.easybeans.deployment.annotations.impl.JInterceptors;
import org.ow2.easybeans.deployment.xml.struct.EJB3;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This class represents the annotation metadata of all classes of an EjbJar
 * file. From this class, we can get metadata of all beans.
 * @author Florent Benoit
 */
public class EjbJarAnnotationMetadata {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(EjbJarAnnotationMetadata.class);

    /**
     * List of class annotations metadata.
     */
    private Map<String, ClassAnnotationMetadata> classesAnnotationMetadata = null;

    /**
     * Link to the Deployment Descriptor object.
     */
    private EJB3 ejb3 = null;

    /**
     * List of application exceptions used on this ejb-jar.
     */
    private Map<String, ApplicationException> applicationExceptions = null;

    /**
     * List of default interceptors by type (business or lifecycle).
     */
    private Map<InterceptorType, List<JClassInterceptor>> defaultInterceptors = null;

    /**
     * List of default interceptors classes.
     */
    private JInterceptors defaultInterceptorsClasses = null;


    /**
     * Constructor.
     */
    public EjbJarAnnotationMetadata() {
        classesAnnotationMetadata = new HashMap<String, ClassAnnotationMetadata>();
    }

    /**
     * Add annotation metadata for a given class.
     * @param classAnnotationMetadata annotation metadata of a class.
     */
    public void addClassAnnotationMetadata(final ClassAnnotationMetadata classAnnotationMetadata) {
        String key = classAnnotationMetadata.getClassName();
        // already exists ?
        if (classesAnnotationMetadata.containsKey(key)) {
            String msg = logger.getI18n().getMessage("EjbJarAnnotationMetadata.addClassAnnotationMetadata.alreadyPresent", key);
            logger.debug(msg);
            throw new IllegalStateException(msg);
        }
        classesAnnotationMetadata.put(key, classAnnotationMetadata);
    }

    /**
     * Gets the class metadata for the given ejb-name.
     * @return class metadata or null if not found
     * @param ejbName the name of the EJB.
     */
    public ClassAnnotationMetadata getClassAnnotationMetadataForEjbName(final String ejbName) {
        for (ClassAnnotationMetadata metadata : classesAnnotationMetadata.values()) {
            JCommonBean bean = metadata.getJCommonBean();
            if (bean != null && ejbName.equals(bean.getName())) {
                return metadata;
            }
        }
        return null;
    }

    /**
     * Get class annotation metadata.
     * @param className key of the map of annotations bean.
     * @return Bean annotation metadata of a given name.
     */
    public ClassAnnotationMetadata getClassAnnotationMetadata(final String className) {
        return classesAnnotationMetadata.get(className);
    }

    /**
     * Get collections of bean annotation metadata.
     * @return collections of bean annotation metadata.
     */
    public Collection<ClassAnnotationMetadata> getClassAnnotationMetadataCollection() {
        return classesAnnotationMetadata.values();
    }

    /**
     * @return the ejb3 deployment descriptor object.
     */
    public EJB3 getEjb3() {
        return ejb3;
    }

    /**
     * Sets the ejb3 deployment descriptor object.
     * @param ejb3 the ejb3 deployment descriptor object.
     */
    public void setEjb3(final EJB3 ejb3) {
        this.ejb3 = ejb3;
    }

    /**
     * Gets the list of application exceptions defined on this ejb jar metadata.
     * @return the list of application exceptions defined on this ejb jar
     *         metadata.
     */
    public Map<String, ApplicationException> getApplicationExceptions() {
        if (applicationExceptions != null) {
            return applicationExceptions;
        }

        // compute it
        applicationExceptions = new HashMap<String, ApplicationException>();

        // For each class, look if it is an application exception
        for (ClassAnnotationMetadata classMetadata : getClassAnnotationMetadataCollection()) {
            ApplicationException appException = classMetadata.getApplicationException();
            // found it then add it in the map.
            if (appException != null) {
                applicationExceptions.put(classMetadata.getClassName().replaceAll("/", "."), appException);
            }
        }

        // Add a default application exception (for checked exception)
        applicationExceptions.put("DEFAULT", new JApplicationException());

        return applicationExceptions;
    }


    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt; (interceptor classes)
     * of default interceptors that enhancer will use.
     */
    public Map<InterceptorType, List<JClassInterceptor>> getDefaultInterceptors() {
        return defaultInterceptors;
    }

    /**
     * Sets the list of default interceptors that enhancers will use.<br>
     * These interceptors are defined by XML DD.
     * @param defaultInterceptors list of interceptors that enhancer will use.
     */
    public void setDefaultInterceptors(final Map<InterceptorType, List<JClassInterceptor>> defaultInterceptors) {
        this.defaultInterceptors = defaultInterceptors;
    }

    /**
     * @return object representing list of default interceptor classes.
     */
    public JInterceptors getDefaultInterceptorsClasses() {
        return defaultInterceptorsClasses;
    }

    /**
     * Sets the object representing the default interceptor classes.
     * @param defaultInterceptorsClasses list of classes
     */
    public void setDefaultInterceptorsClasses(final JInterceptors defaultInterceptorsClasses) {
        this.defaultInterceptorsClasses = defaultInterceptorsClasses;
    }


}
