/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ENCBindingBuilder.java 2067 2007-11-27 10:03:12Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.enc;

import java.util.Arrays;
import java.util.List;

import org.ow2.easybeans.asm.Type;
import org.ow2.easybeans.deployment.annotations.JMethod;
import org.ow2.easybeans.deployment.annotations.impl.JAnnotationResource;
import org.ow2.easybeans.deployment.annotations.impl.JavaxPersistenceContext;
import org.ow2.easybeans.deployment.annotations.impl.JavaxPersistenceUnit;
import org.ow2.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.ow2.easybeans.deployment.annotations.metadata.CommonAnnotationMetadata;
import org.ow2.easybeans.deployment.annotations.metadata.EjbJarAnnotationMetadata;
import org.ow2.easybeans.deployment.annotations.metadata.FieldAnnotationMetadata;
import org.ow2.easybeans.deployment.annotations.metadata.MethodAnnotationMetadata;
import org.ow2.easybeans.deployment.api.IJAnnotationResource;
import org.ow2.easybeans.deployment.api.IJavaxPersistenceContext;
import org.ow2.easybeans.deployment.api.IJavaxPersistenceUnit;

/**
 * Utility class that helps to build the ENC Binding by analyzing metadata.
 * @author Florent BENOIT
 */
public class ENCBindingBuilder {

    /**
     * All class metadata are kept by this object.
     */
    private EjbJarAnnotationMetadata ejbJarAnnotationMetadata = null;

    /**
     * Binding holder object.
     */
    private ENCBindingHolder bindingHolder = null;

    /**
     * Replace length to create default JNDI names.
     */
    private static final int GETTER_LENGTH = 3;

    /**
     * Constructor used only by this class/subclasses.
     * @param ejbJarAnnotationMetadata the given metadata to analyze.
     */
    protected ENCBindingBuilder(final EjbJarAnnotationMetadata ejbJarAnnotationMetadata) {
        this.ejbJarAnnotationMetadata = ejbJarAnnotationMetadata;
        this.bindingHolder = new ENCBindingHolder();
    }

    /**
     * Analyze the metadata and populate the binding holder.
     * @throws ENCBindingException if the analyze is failing.
     */
    protected void analyze() throws ENCBindingException {

        // Analyze each class
        for (ClassAnnotationMetadata classMetadata : ejbJarAnnotationMetadata.getClassAnnotationMetadataCollection()) {
            analyzeClassMetadata(classMetadata);
        }
    }

    /**
     * Analyze the given class metadata.
     * @param classMetadata the given metadata to analyze.
     * @throws ENCBindingException if the class has not been analyzed
     */
    protected void analyzeClassMetadata(final ClassAnnotationMetadata classMetadata) throws ENCBindingException {
        // Are there persistence context (only on Class)
        List<JavaxPersistenceContext> javaxPersistenceContexts = classMetadata.getJavaxPersistencePersistenceContexts();
        if (javaxPersistenceContexts != null) {
            for (JavaxPersistenceContext javaxPersistenceContext : javaxPersistenceContexts) {
                addPersistenceContext(javaxPersistenceContext);
            }
        }

        // Are there persistence unit (only on Class)
        List<JavaxPersistenceUnit> javaxPersistenceUnits = classMetadata.getJavaxPersistencePersistenceUnits();
        if (javaxPersistenceUnits != null) {
            for (JavaxPersistenceUnit javaxPersistenceUnit : javaxPersistenceUnits) {
                addPersistenceUnit(javaxPersistenceUnit);
            }
        }

        // Are there resource (only on Class)
        List<JAnnotationResource> jAnnotationResources = classMetadata.getJAnnotationResources();
        if (jAnnotationResources != null) {
            for (JAnnotationResource jAnnotationResource : jAnnotationResources) {
                addResource(jAnnotationResource);
            }
        }

        // Analyze common stuff on the class
        analyzeCommonMetadata(classMetadata);

        // Then, analyze each field of the class
        for (FieldAnnotationMetadata fieldAnnotationMetadata : classMetadata.getFieldAnnotationMetadataCollection()) {
            analyzeFieldMetadata(fieldAnnotationMetadata);
        }

        // And each method of the class
        for (MethodAnnotationMetadata methodAnnotationMetadata : classMetadata.getMethodAnnotationMetadataCollection()) {
            analyzeMethodMetadata(methodAnnotationMetadata);
        }

    }

    /**
     * Analyze the given field metadata.
     * @param fieldMetadata the given metadata to analyze.
     * @throws ENCBindingException if the field has not been analyzed
     */
    protected void analyzeFieldMetadata(final FieldAnnotationMetadata fieldMetadata) throws ENCBindingException {
        // Analyze common stuff on the field
        analyzeCommonMetadata(fieldMetadata);
    }

    /**
     * Analyze the given method metadata.
     * @param methodAnnotationMetadata the given metadata to analyze.
     * @throws ENCBindingException if the method has not been analyzed
     */
    protected void analyzeMethodMetadata(final MethodAnnotationMetadata methodAnnotationMetadata) throws ENCBindingException {
        // Analyze common stuff on the method
        analyzeCommonMetadata(methodAnnotationMetadata);
    }

    /**
     * Analyze the given common metadata.
     * @param commonAnnotationMetadata the given metadata to analyze.
     * @throws ENCBindingException if the common metadata has not been analyzed
     */
    protected void analyzeCommonMetadata(final CommonAnnotationMetadata commonAnnotationMetadata) throws ENCBindingException {
        JavaxPersistenceContext javaxPersistenceContext = commonAnnotationMetadata.getJavaxPersistenceContext();
        if (javaxPersistenceContext != null) {
            // Set name if null
            if (isNullOrEmpty(javaxPersistenceContext.getName())) {
                javaxPersistenceContext.setName(getName(commonAnnotationMetadata, javaxPersistenceContext));
            }
            addPersistenceContext(javaxPersistenceContext);
        }

        JavaxPersistenceUnit javaxPersistenceUnit = commonAnnotationMetadata.getJavaxPersistenceUnit();
        if (javaxPersistenceUnit != null) {
            // Set name if null
            if (isNullOrEmpty(javaxPersistenceUnit.getName())) {
                javaxPersistenceUnit.setName(getName(commonAnnotationMetadata, javaxPersistenceUnit));
            }
            addPersistenceUnit(javaxPersistenceUnit);
        }

        IJAnnotationResource jAnnotationResource = commonAnnotationMetadata.getJAnnotationResource();
        if (jAnnotationResource != null) {
            // Set name if null
            if (isNullOrEmpty(jAnnotationResource.getName())) {
                jAnnotationResource.setName(getName(commonAnnotationMetadata, jAnnotationResource));
            }
            jAnnotationResource.setType(getType(commonAnnotationMetadata, jAnnotationResource));
            addResource(jAnnotationResource);
        }

    }

    /**
     * Checks if the given string is empty or null.
     * @param name the given string to test
     * @return true if the string is null or empty
     */
    private boolean isNullOrEmpty(final String name) {
        return (name == null) || "".equals(name);
    }

    /**
     * Gets the ENC name for the given metadata.
     * @param commonAnnotationMetadata the metadata to analyze.
     * @param o the given object associated to the metadata (used for logging).
     * @return the given ENC name
     * @throws ENCBindingException if no name has been defined on the given name
     *         and the metadata is invalid.
     */
    private String getName(final CommonAnnotationMetadata commonAnnotationMetadata, final Object o) throws ENCBindingException {
        if (commonAnnotationMetadata instanceof FieldAnnotationMetadata) {
            FieldAnnotationMetadata fieldAnnotationMetadata = (FieldAnnotationMetadata) commonAnnotationMetadata;
            return fieldAnnotationMetadata.getClassAnnotationMetadata().getClassName().replace("/", ".") + "/"
                    + fieldAnnotationMetadata.getFieldName();
        }

        if (commonAnnotationMetadata instanceof MethodAnnotationMetadata) {
            MethodAnnotationMetadata methodAnnotationMetadata = (MethodAnnotationMetadata) commonAnnotationMetadata;

            StringBuilder propertyBuilder = new StringBuilder(methodAnnotationMetadata.getMethodName());
            propertyBuilder.delete(0, GETTER_LENGTH);
            propertyBuilder.setCharAt(0, Character.toLowerCase(propertyBuilder.charAt(0)));
            propertyBuilder.insert(0, methodAnnotationMetadata.getClassAnnotationMetadata().getClassName().replace("/", ".")
                    + "/");
            return propertyBuilder.toString();
        }

        // Last case is the Class case.
        throw new ENCBindingException("There is no defined name in the object '" + o + "' that is defined on the class '"
                + commonAnnotationMetadata + "'.");

    }

    /**
     * Gets the type for the given metadata.
     * @param commonAnnotationMetadata the metadata to analyze.
     * @param jAnnotationResource the given annotation resouce object
     * @return the given type value (interface)
     * @throws ENCBindingException if no name has been defined on the given name
     *         and the metadata is invalid.
     */
    private String getType(final CommonAnnotationMetadata commonAnnotationMetadata,
            final IJAnnotationResource jAnnotationResource) throws ENCBindingException {
        if (commonAnnotationMetadata instanceof FieldAnnotationMetadata) {
            FieldAnnotationMetadata fieldAnnotationMetadata = (FieldAnnotationMetadata) commonAnnotationMetadata;
            Type typeInterface = Type.getType(fieldAnnotationMetadata.getJField().getDescriptor());
            return typeInterface.getClassName();
        } else if (commonAnnotationMetadata instanceof MethodAnnotationMetadata) {
            MethodAnnotationMetadata methodAnnotationMetadata = (MethodAnnotationMetadata) commonAnnotationMetadata;

            JMethod jMethod = methodAnnotationMetadata.getJMethod();
            Type[] args = Type.getArgumentTypes(jMethod.getDescriptor());
            if (args.length != 1) {
                throw new ENCBindingException("Method args '" + Arrays.asList(args) + "' for method '" + jMethod
                        + "' are invalid. Length should be of 1.");
            }
            Type typeInterface = args[0];
            return typeInterface.getClassName();
        }

        return jAnnotationResource.getType();
    }

    /**
     * Adds the given persistence context to the list of bindings.
     * @param javaxPersistenceContext the given persistence context.
     */
    protected void addPersistenceContext(final IJavaxPersistenceContext javaxPersistenceContext) {
        bindingHolder.addPersistenceContextBinding(new ENCBinding<IJavaxPersistenceContext>(javaxPersistenceContext.getName(),
                javaxPersistenceContext));
    }

    /**
     * Adds the given persistence unit to the list of bindings.
     * @param javaxPersistenceUnit the given persistence unit.
     */
    protected void addPersistenceUnit(final IJavaxPersistenceUnit javaxPersistenceUnit) {
        bindingHolder.addPersistenceUnitBinding(new ENCBinding<IJavaxPersistenceUnit>(javaxPersistenceUnit.getName(),
                javaxPersistenceUnit));
    }

    /**
     * Adds the given resource to the list of bindings.
     * @param jAnnotationResource the given resource.
     */
    protected void addResource(final IJAnnotationResource jAnnotationResource) {
        bindingHolder.addResourceBinding(new ENCBinding<IJAnnotationResource>(jAnnotationResource.getName(),
                jAnnotationResource));
    }

    /**
     * @return the Holder of all ENC bindings.
     */
    protected ENCBindingHolder getHolder() {
        return bindingHolder;
    }

    /**
     * Found all ENC bindings from the given set of metadata.
     * @param ejbJarAnnotationMetadata the given metadata to analyze.
     * @return object that contains all bindings found for the given set of
     *         classes.
     * @throws ENCBindingException if the analyze of the set of the metadata has
     *         failed.
     */
    public static ENCBindingHolder analyze(final EjbJarAnnotationMetadata ejbJarAnnotationMetadata) throws ENCBindingException {
        ENCBindingBuilder bindingBuilder = new ENCBindingBuilder(ejbJarAnnotationMetadata);
        bindingBuilder.analyze();
        return bindingBuilder.getHolder();
    }
}
